from megalus.command.base import DockerBase
from megalus.core.utils import locale as _
from buzio import console


class Up(DockerBase):

    def run(self):
        console.section(_("Docker Up"), transform="center", full_width=True)

        # Get from name if informed
        name = ""
        container_id = None
        if self.arguments["<application>"]:
            container_id, name = self._get_application()
            if not name:
                return False

        if name:
            if container_id:
                ret = self._run_command(
                    task='docker stop {}'.format(container_id)
                )
            ret = self._run_command(
                task="cd {folder} && docker-compose up -d {name}".format(
                    folder=self.project['dc_path'],
                    name=name
                )
            )
        else:
            # Stop all containers
            ret = self._stop_all_containers()
            if not ret:
                return False
            ret = self._run_command(
                task="cd {folder} && docker-compose up".format(
                    folder=self.project['dc_path']
                )
            )
        return True



import platform
from megalus.command.base import DockerBase
from megalus.core.utils import locale as _
from buzio import console


class Stop(DockerBase):

    def run(self):
        console.section(_("Docker Stop"), transform="center", full_width=True)

        ret = self._stop_all_containers()
        if not ret:
            return False
        self._remove_extra_containers()
        if self.arguments['--force']:
            ret = self._run_command(
                task="docker ps -q"
            )
            if ret:
                console.warning(_("Force stop frozen containers"))
                self._run_command(
                    task="docker update --restart=no $(docker ps -q)"
                )
                if platform.system().lower() == "linux":
                    self._run_command(
                        task="systemctl restart docker"
                    )
                else:
                    console.success("Please restart Docker to complete operation.")
        return True




import platform
from megalus.command.base import DockerBase
from megalus.core.utils import locale as _
from buzio import console


class Run(DockerBase):

    def run(self):
        console.section(_("Docker Run"), transform="center", full_width=True)

        # Get from name if informed
        name = ""
        container_id = None
        if self.arguments["<application>"]:
            container_id, name = self._get_application()
            if not name:
                return False

        if container_id:
            if self.arguments['<command>']:
                self._run_command(
                    task="docker exec -ti {container} {command}".format(
                        container=container_id,
                        command=self.arguments['<command>']
                    ),
                    verbose=True
                )
            else:
                self._run_service(name)
        else:
            if self.arguments['<command>']:
                self._run_command(
                    task="cd {path} && docker-compose "
                    "run --rm --service-ports {name} {command}".format(
                        path=self.project['dc_path'],
                        name=name,
                        command=self.arguments['<command>']),
                    verbose=True)
            else:
                self._run_service(name)

    def _run_service(self, name):
        data = self._get_application_info(name)
        if not data:
            console.error(_("Can't find data for {}".format(name)))
            return False
        if not data['State']['Running']:
            ret = self._run_command(
                task="cd {folder} && docker-compose up -d {name}".format(
                    folder=self.project['dc_path'],
                    name=name
                )
            )
            if not ret:
                console.error(_("Can't start {}".format(name)))
                return False

        service = [
            obj
            for obj in self.project['dc_applications']
            if obj['name'] == name
        ][0]
        if service['type'].name in ['Backend', 'Frontend', 'DevOps']:
            port = service['ports'][0]
            if ":" in port:
                url = "http://localhost:{}".format("".join(port.split(":")[0]))
                if platform.system().lower() != 'linux' \
                        and platform.system().lower() != 'windows':
                    command = "open -a " + \
                        self.project['browser'].get_command_name()
                else:
                    command = self.project['browser'].get_command_name
                open_command = command.format(url=url)
                self._run_command(
                    task=open_command,
                    verbose=True
                )
                return True
        console.warning('No startup command found for {}'.format(name))



import os
import pathlib
import re
from buzio import console
from megalus.command.base import DockerBase
from megalus.core.message import notify
from megalus.core.utils import locale as _


class Create(DockerBase):

    def run(self, delete_data=False):

        console.clear()
        console.section(
            _("Create {} Project".format(self.project['name'])),
            transform="center",
            full_width=True)

        ret = console.confirm()
        if not ret:
            return False

        # Stop all containers
        self._stop_all_containers()

        if delete_data or console.confirm(
                "Delete all Docker data before cloning"):
            self._run_command(
                title=_("Excluding docker data"),
                task="cd {folder} && docker-compose down".format(
                    folder=self.project['dc_path'])
            )

        for application in self.project['dc_applications']:
            if application['base'] == "image":
                continue

            console.section(_("Application: {}".format(application['name'])))
            path = application['path']
            if "$" in path:
                s = re.search("\${(\w+)}", application['path'])
                if s:
                    env = s.group(1)
                    name = os.environ.get(env)
                    path_list = [
                        part if "$" not in part else name
                        for part in application['path'].split("/")
                    ]
                    path = os.path.join(path_list)
                else:
                    console.error(
                        "Cant find path for {}".format(
                            application['name']))
                    return False
            if "." in path:
                list_path = os.path.join(self.project['dc_path'], path)
                path = os.path.abspath(list_path)

            console.info(_("Working directory: {}".format(path)),
                         use_prefix=False)
            pathlib.Path(path).mkdir(parents=True, exist_ok=True)
            if not os.listdir(path):
                ret = self._run_command(
                    title=_("Cloning {}".format(application['repo'])),
                    task="cd {} && git clone {} .".format(
                        path,
                        application['repo']
                    )
                )
            else:
                console.info(
                    _("Dir exists. Skipping...".format(path)))
